#include <qpe/qpeapplication.h>
#include <qpe/resource.h>
#include <qpe/global.h>
#include <qpe/qpetoolbar.h>
#include <qpe/qpemenubar.h>
#include <qmainwindow.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlistview.h>
#include <qstring.h>
#include <qlayout.h>
#include <qaction.h>
#include <qdialog.h>
#include <qpopupmenu.h>
#include <qtextview.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qregexp.h>
#include <qtimer.h>

#include <stdlib.h>
#include <sqlite.h>

#include "drug.h"
#include "infowidget.h"
#include "note.h"

int searchResults=0;
int fontSize = 0;

int indexCallback(void *pArg, int, char **argv, char **)
{
    QListView *results=(QListView *)pArg;
    QListViewItem *item = new QListViewItem(results, argv[0]);
    results->insertItem( item );
    return 0;
}

int recentDrugsCallback(void *pArg, int, char **argv, char **)
{
    drug *drg=(drug *)pArg;
    QString drug=argv[0];
    if(drug.length() > 40)
        drug.replace(40,3,"...");
    QAction *a = new QAction( drug.left(43), QString::null,
                            0, 0, argv[0]  );
    QObject::connect(a, SIGNAL(activated()), drg, SLOT(slotOpenRecent()));
    a->addTo(drg->recentPopup);
    
    return 0;
}

int charCallback(void *pArg, int, char **argv, char **)
{
    QListView *results=(QListView*)pArg;
    QListViewItem *item = new QListViewItem(results, !argv ? "No results" : argv[0]);
    results->insertItem(item);
    return 0;
}

int searchCallback(void *pArg, int, char **argv, char **)
{
    QListView *results=(QListView*)pArg;
    QListViewItem *item = new QListViewItem(results, argv==0 ? "No results" : argv[0]);
    results->insertItem(item);
    searchResults++;
    return 0;
}

int fontSizeCallback(void *, int, char **argv, char **)
{
    QString str(argv[0]);
    fontSize = str.toInt();
    return 0;
}

int abbreviationsCallback(void *pArg, int, char **argv, char **)
{
    QMultiLineEdit *lineEdit =(QMultiLineEdit *)pArg;
    lineEdit->setText(argv[0]);
    return 0;
}

DisclaimerDialog::DisclaimerDialog(QWidget *parent, const char *name)
                       : QDialog(parent, name, TRUE)
{
    setCaption("Medical Disclaimer");
    QVBoxLayout *vlay = new QVBoxLayout(this);
    vlay->setSpacing(4);
    QTextView *disclaimer = new QTextView(this);
    disclaimer->setText("<div align=\"center\"><b>Disclaimer</b></div>"
    				                  "This application is designed for general <br>"
                                                      "informational purposes only.<br>"
						      "The Contents of the database may not be<br>"
						      "100% accurate and THE AUTHOR IS NOT<br>"
						      "RESPONSIBLE OR LIABLE FOR ANY<br>"
						      "INFORMATION that you obtain throgh<br>"
						      "this software.<br>"
						      "Reliance on any information on this<br>"
						      "application is solely at YOUR OWN RISK");
    vlay->addWidget(disclaimer);
    QHBoxLayout *hlay = new QHBoxLayout(vlay);
    QPushButton *ok = new QPushButton("OK", this);
    connect(ok, SIGNAL(clicked()),SLOT(accept()));
    hlay->addWidget(ok);
    QPushButton *quit = new QPushButton("Quit", this);
    connect(quit, SIGNAL(clicked()),SLOT(reject()));
    hlay->addWidget(quit);
    resize(227, 190);
}

AboutDialog::AboutDialog(QWidget *parent, const char *name)
                 : QDialog(parent,name,TRUE)
{
    QVBoxLayout *vbox=new QVBoxLayout(this);
    vbox->setMargin(5);
    vbox->setSpacing(3);
    QHBoxLayout *hbox=new QHBoxLayout(vbox);
    hbox->setSpacing(3);
    QLabel *img = new QLabel(this);
    img->setPixmap(Resource::loadPixmap("txdrug"));
    hbox->addWidget(img);
    hbox->addSpacing(4);
    QLabel *app = new QLabel(this);
    app->setText("<font size=\"+1\" ><b>TxDrug<b></font><br>");
    hbox->addWidget(app);
    hbox->addStretch();
    QLabel *version = new QLabel("<b>Version:</b> 0.1", this);
    version->setAlignment(Qt::AlignLeft);
    vbox->addWidget(version);
    QLabel *author = new QLabel("<b>Author:</b> Pierpaolo Di Panfilo<br>"
                                                "<b>E-mail:</b> pippo_dp@libero.it", this);
    author->setAlignment(Qt::AlignLeft);
    vbox->addWidget(author);
    QLabel *description = new QLabel("<b>Description:</b><br>Drug Lookup for medical professional",this);
    description->setAlignment(Qt::AlignLeft);
    vbox->addWidget(description);
    vbox->addStretch();
    QPushButton *ok = new QPushButton("OK",this);
    vbox->addWidget(ok);
    connect(ok, SIGNAL(clicked()), SLOT(accept()));
    resize(200, 150);
}

DrugList::DrugList(QWidget *parent)
            : QListView(parent)
{
    ((QWidget*)this)->setMinimumWidth(180);
    addColumn(tr("Main Index"), width());
    setAllColumnsShowFocus( TRUE );
}

void DrugList::keyPressEvent( QKeyEvent *e )
{
    QString txt = e->text();
    if (e->key() == Key_Space)
        emit returnPressed( currentItem() );
    else if ( !txt.isNull() && txt[0] > ' ' && e->key() < 0x1000 )
        e->ignore();
    else
        QListView::keyPressEvent(e);
}


drug::drug(QWidget *parent, const char* name, WFlags f)
     : QMainWindow (parent, name, f)
{
      setCaption(tr("TxDrug"));
      setIcon(Resource::loadPixmap("txdrug"));
      char *errmsg;
     if( (sql=sqlite_open("/opt/QtPalmtop/etc/txdrug/drugs.db",0666,&errmsg))==NULL )
     {
        QString err=errmsg;
        QMessageBox::critical(0,"Drug Lookup","Error opening the database:\n"+err);
     }
     
    recentPopup = new QPopupMenu(this);
    setToolBarsMovable( FALSE );
         
    QPEToolBar *bar=new QPEToolBar(this);
    bar->setHorizontalStretchable( TRUE );
    
    QPEMenuBar *menubar = new QPEMenuBar(bar);
    menubar->insertItem("Recent drugs", recentPopup);
    noteMenu = new NotePopup(this, sql);
    menubar->insertItem("Note", noteMenu);
        
    bar = new QPEToolBar( this );

    QAction *a = new QAction( tr( "New Note" ), Resource::loadPixmap( "new" ),
                              QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ),
             this, SLOT( slotCreateNote() ) );
    a->addTo( bar );
    
    QPopupMenu *helpMenu = new QPopupMenu(this);
    a = new QAction( tr( "Abbreviations used" ), QString::null, 0, 0  );
    connect( a, SIGNAL( activated() ),this, SLOT( slotAbbreviations() ) );
    a->addTo( helpMenu );
    helpMenu->insertSeparator();
    a = new QAction( tr( "About" ), QString::null, 0, 0  );
    connect( a, SIGNAL( activated() ),this, SLOT( slotAbout() ) );
    a->addTo( helpMenu );
    
    QToolButton *help = new QToolButton(Resource::loadPixmap("help_icon"), 0, 0, 0, 0,bar);
    help->setPopup(helpMenu);
    help->setPopupDelay(1);
    
    QVBox *vbox = new QVBox(this);
    vbox->setSpacing(5);
    vbox->setMargin(5);
    
    QHBox *searchBox = new QHBox(vbox);
    searchBox->setSpacing(2);
    term = new QLineEdit(searchBox);
    incombo = new QComboBox(searchBox);
    incombo->insertItem("Name");
    incombo->insertItem("Indication");
    incombo->insertItem("Pharma");
    incombo->insertItem("Category");
    search = new QPushButton(Resource::loadPixmap("find"), tr("Go"), searchBox);
    connect(search, SIGNAL(clicked()), SLOT(slotSearchPressed()));
    
    QHBox *hbox = new QHBox(vbox);
    results = new DrugList(hbox);
    char *alfabetico[]={"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};
    QVBox *rightBox=new QVBox(hbox);
    ((QWidget*)rightBox)->setSizePolicy(QSizePolicy(QSizePolicy::Maximum,QSizePolicy::Maximum));
    ((QWidget*)rightBox)->setMaximumWidth(23);
    rindex=new QPushButton("dsd", rightBox);
    rindex->setPixmap(Resource::loadPixmap("back"));
    connect(rindex,SIGNAL(clicked()),SLOT(slotReturnIndex()));
    sv = new QScrollView(rightBox);
    sv->setVScrollBarMode(QScrollView::AlwaysOff);
    sv->setHScrollBarMode(QScrollView::AlwaysOff);
    QVBox *charBox = new QVBox(sv);
    for(int i=0; i<26;i++)
    {
        group[i]=new QPushButton(alfabetico[i],charBox);
        group[i]->setToggleButton(true);
        group[i]->setFlat(true);
        ((QWidget*)group[i])->setMinimumWidth(23);
        connect(group[i], SIGNAL(toggled(bool)), SLOT(charToggled(bool)));
        
    }
    sv->addChild(charBox);
    QPushButton *b=new QPushButton(rightBox);
    b->setPixmap(Resource::loadPixmap("txdrug/up.png"));
    ((QWidget*)b)->setMinimumHeight(10);
    connect(b,SIGNAL(pressed()),SLOT(slotScrollUp()));
    connect(b,SIGNAL(released()),SLOT(slotReleased()));
    b=new QPushButton(rightBox);
    b->setPixmap(Resource::loadPixmap("txdrug/down.png"));
    ((QWidget*)b)->setMinimumHeight(10);
    connect(b,SIGNAL(pressed()),SLOT(slotScrollDown()));
    connect(b,SIGNAL(released()),SLOT(slotReleased()));
    connect(results, SIGNAL(clicked(QListViewItem *)),SLOT(slotItemClicked(QListViewItem *)));
    connect(results, SIGNAL(returnPressed(QListViewItem *)),SLOT(slotItemClicked(QListViewItem *)));
    hbox->setStretchFactor(results, 10);
    hbox->setStretchFactor(rightBox, 1);
    results->setColumnWidth(0,results->width());
    setCentralWidget( vbox );
        
    showIndex();
    loadRecentDrugs();
    loadFontSize();
    released=false;
}

void drug::closeEvent( QCloseEvent *e )
{
    QString size=QString::number(fontSize);
    char *errmsg;
    if(sqlite_exec(sql,"UPDATE font SET fontsize="+size,0,0,&errmsg)!=SQLITE_OK )
    {
        QString err=errmsg;
        QMessageBox::warning(0,"TxDrug  ","Error saving font size:\n"+err);
    }
    e->accept();
}

void drug::loadFontSize()
{
    char *errmsg;
    if(sqlite_exec(sql, "SELECT fontsize FROM font",
            fontSizeCallback, 0, &errmsg)!=SQLITE_OK )
    {
        QString err=errmsg;
        QMessageBox::warning(0,"TxDrug  ","Error loadind font size:\n"+err);
    }
}

void drug::loadRecentDrugs()
{
    uint count=((QMenuData *)recentPopup)->count();
    for(uint i=0;i<count;i++)
        recentPopup->removeItemAt(0);
    
    char *errmsg;
    if(sqlite_exec(sql, "SELECT * FROM recent ORDER BY num DESC",
            recentDrugsCallback, this, &errmsg)!=SQLITE_OK )
    {
        QString err=errmsg;
        QMessageBox::warning(0,"TxDrug  ","Error loading recent drugs:\n"+err);
    }
            
}

void drug::slotCreateNote()
{
    NoteDialog(this, -1, -1, sql);
    noteMenu->loadNote();
}

void drug::slotAbbreviations()
{
    QDialog *dialog = new QDialog(this);
    dialog->setCaption("Abbreviations in Drug Information");
    QVBoxLayout *vbox = new QVBoxLayout(dialog);
    QMultiLineEdit *multiLine = new QMultiLineEdit(dialog);
    char *errmsg;
    if (sqlite_exec(sql,"SELECT abbr FROM abbreviations", abbreviationsCallback, 
                          multiLine, &errmsg)!=SQLITE_OK)
        QMessageBox::warning(dialog, "TxDrug","Error loading abbreviations:\n"+QString(errmsg),0);
    
    vbox->addWidget(multiLine);
#if defined(Q_WS_QWS) || defined(_WS_QWS_)
    dialog->showMaximized();
#endif
    dialog->exec();
}

void drug::slotAbout()
{
    AboutDialog *about = new AboutDialog(this);
    about->exec();
}

void drug::slotReleased()
{
    released=true;
}

void drug::slotScrollUp()
{
    released=false;
    sv->scrollBy(0,-20);
    t=new QTimer();
    connect(t,SIGNAL(timeout()),SLOT(slotFastScroll()));
    t->start(800,FALSE);
    up=true;
}

void drug::slotScrollDown()
{
    released=false;
    sv->scrollBy(0,20);
    t=new QTimer();
    connect(t,SIGNAL(timeout()),SLOT(slotFastScroll()));
    t->start(800,FALSE);
    up=false;
}

void drug::stopAutoScroll()
{
    delete t;
    t=0;
}

void drug::slotFastScroll()
{
if ( !released ){
        if ( t )
    {
            t->changeInterval( 50 );
        up ? sv->scrollBy(0,-20) : sv->scrollBy(0,20);
    }
 } else {
        stopAutoScroll();
    }
}

void drug::clearListView()
{
    QListViewItemIterator it(results);
    for(; it.current(); )
        results->takeItem(it.current());
}

void drug::showIndex()
{
    char *errmsg;
    if(sqlite_exec(sql,"SELECT name FROM drugs",indexCallback,results,&errmsg)!=SQLITE_OK )
    {
        QString err=errmsg;
        QMessageBox::warning(0,"TxDrug  ","Error loading drugs:\n"+err);
    }
    results->setColumnText(0,"Main Index");
    rindex->hide();
}

void drug::slotReturnIndex()
{
    clearListView();
    showIndex();
    for(int i=0; i<26;i++)
    {
        if(group[i]->isOn())
            group[i]->setOn(false);
    }
}

void drug::searchDrug(QString &term)
{
   QListViewItemIterator it(results);
    for(; it.current(); )
        results->takeItem(it.current());
        
    QString by=incombo->currentText();
    if(by=="Indication")
        by="indic";
    else if(by=="Pharma")
        by="pharm";
    else if(by=="Category")
        by="bnf";
    char *errmsg;
    term.replace(QRegExp("'"),"''");
    if (sqlite_exec(sql,"SELECT name FROM drugs WHERE " + by + " like '%" + term + "%'",
            searchCallback,results,&errmsg)!=SQLITE_OK )
    {
        QString err=errmsg;
        QMessageBox::warning(0,"TxDrug  ","Error searching drug:\n"+err);
    }
    results->setColumnText(0,"Search Results");
    if(!searchResults)
    {
        QListViewItem *item = new QListViewItem(results,"No Results");
        item->setSelectable(FALSE);
        results->insertItem(item);
    }
    searchResults=0;
    rindex->show();
}

void drug::charToggled(bool on)
{
    QPushButton *a=(QPushButton *)sender();
    QString ch=a->text();
    if(!on)
    {
        bool one=true;
    
        for(int i=0; i<26;i++)
        {
            if(group[i]->isOn())
                one=false;
        }
        clearListView();
        if(one)
            showIndex();
        return;
    }
    
    for(int i=0; i<26;i++)
    {
        if(group[i]->isOn()  && group[i]!=a)
            group[i]->setOn(false);
    }
    clearListView();
    char *errmsg;
    if(sqlite_exec(sql,"SELECT name FROM drugs WHERE name like '"+ch+"%'",
            charCallback,results,&errmsg)!=SQLITE_OK )
    {
        QString err=errmsg;
        QMessageBox::warning(0,"TxDrug  ","Error:\n"+err);
    }
    results->setColumnText(0,ch+" Index");
    rindex->show();
    
}

void drug::slotSearchPressed()
{
    QString text = term->text();
    searchDrug(text);
}

void drug::slotOpenRecent()
{
    QAction *a=(QAction *)sender();
    QString name=a->name();
    (void)new InfoDialog(this, name, sql);
}

void drug::slotItemClicked(QListViewItem *item)
{
    if( item == NULL)
       return;
    QString name=item->text(0);
    if(name=="No Results")
        return;

    InfoDialog *infoDialog = new InfoDialog(this,name, sql);
    infoDialog->exec();
        
    if(recentPopup->idAt(8) != -1)
    {
        char *errmsg;
        if(sqlite_exec(sql,"DELETE FROM recent WHERE num=(SELECT MIN(num) FROM recent)",
                            0,0,&errmsg)!=SQLITE_OK )
        {
            QString err=errmsg;
            QMessageBox::warning(0,"TxDrug  ","Error deleting:\n"+err);
        }
    }        

    char *errmsg;
    name.replace(QRegExp("'"), "''");
    
    if(sqlite_exec(sql,"INSERT INTO recent (drug) values('" + name + "')",0,0,&errmsg)!=SQLITE_OK )
        {
            QString err=errmsg;
            QMessageBox::warning(0,"TxDrug  ","Error inserting:\n"+err);
        }
    loadRecentDrugs();
    noteMenu->loadNote();
}


