#ifndef DRUG_H
#define DRUG_H

#include <qmainwindow.h>
#include <qdialog.h>
#include <qlistview.h>
#include <sqlite.h>
#include <qdict.h>

class QPopupMenu;
class NotePopup;
class QListViewItem;
class QString;
class QLineEdit;
class QPushButton;
class QScrollView;
class QComboBox;
class QTimer;

//callbacks for sqlite_exec
int indexCallback(void *pArg, int argc, char **argv, char **columnNames);
int recentDrugsCallback(void *pArg, int argc, char **argv, char **columnNames);
int charCallback(void *pArg, int argc, char **argv, char **columnNames);
int searchCallback(void *pArg, int, char **argv, char **);
int fontSizeCallback(void *pArg, int, char **argv, char **);
int abbreviationsCallback(void *pArg, int, char **argv, char **);

class DisclaimerDialog : public QDialog
{
    Q_OBJECT
public:
    DisclaimerDialog(QWidget *parent, const char *name=0);    
};

class AboutDialog : public QDialog
{
    Q_OBJECT
public:
    AboutDialog(QWidget *parent, const char *name=0);
};

class DrugList : public QListView
{
    Q_OBJECT
public:
    DrugList(QWidget *parent);
    
protected:
    void keyPressEvent( QKeyEvent *e );
};

class drug : public QMainWindow
{
    Q_OBJECT
public:
    drug( QWidget *parent = 0, const char *name = 0, WFlags f = 0 );
    QPopupMenu *recentPopup;
    QPushButton *rindex;
        
protected:
    void closeEvent( QCloseEvent *e );
    
private slots:
    void slotCreateNote();
    void slotAbout();
    void slotAbbreviations();
    void slotSearchPressed();
    void slotReturnIndex();
    void slotOpenRecent();
    void slotItemClicked(QListViewItem *item);
    void loadRecentDrugs();
    void slotScrollUp();
    void slotScrollDown();
    void slotReleased();
    void slotFastScroll();
    void stopAutoScroll();
    void charToggled(bool on);
    
private:
    void clearListView();
    void showIndex();
    void loadFontSize();
    void searchDrug(QString& term);
    void loadFile( const QString& filename);
    
    QLineEdit *term;
    QPushButton *search;
    QListView *results;
    QScrollView *sv;
    QComboBox *incombo;
    NotePopup *noteMenu;
    QPushButton *group[25];
    QPushButton *precautions;
    QDict< QDict<QString> > gDict;
    bool released;
    QTimer *t;
    bool up;
    sqlite *sql;
};

#endif
