#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qmultilineedit.h>
#include <qdialog.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qaction.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpe/resource.h>
#include <qpe/qpemessagebox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qstring.h>
#include "note.h"

int noteCallback(void *pArg, int, char **argv, char **)
{
	QMultiLineEdit *note=(QMultiLineEdit *)pArg;
	note->setText(argv[0]);
	return 0;
}

int loadNoteCallback(void *pArg, int, char **argv, char **)
{	
	NotePopup *np=(NotePopup *)pArg;
	QString str(argv[1]);
	int drugId=str.toInt();
	str = argv[2];
	int noteId=str.toInt();
	
	if(drugId<0)
	{
	qWarning("in loadNoteCallback noteid=%d title=%s", noteId,argv[0]);
	    np->generalPopup()->insertItem(argv[0], np, SLOT(slotNote(int)), 0, noteId);
	} else
	    np->addDrugNote(drugId, argv[0]);
	return 0;
}

int drugNameCallback(void *pArg, int, char **argv, char **)
{
    NotePopup *np=(NotePopup *)pArg;
    QString drug(argv[0]);
    QString title = np->drugNoteTitle();
    
    if(drug.length() > 12)
        drug.replace(12,2,"..");
    QString text("["+drug.left(14)+"] "+title);
    
    QString str(argv[1]);
    int drugId= str.toInt();
    np->drugPopup()->insertItem(text, np, SLOT(slotDrugNote(int)), 0, drugId);
    
    return 0;
}

int loadDrugNoteCallback(void *pArg, int , char **argv, char **)
{
    NoteDialog *noteDialog = (NoteDialog *)pArg;
    if(argv[0]!="")
        noteDialog->setEdit(TRUE,argv[0], argv[1], argv[2] ? argv[2] : 0);
    else noteDialog->setEdit(FALSE);
    qWarning(argv[2]);
    return 0;
}

NoteDialog::NoteDialog(QWidget *parent, int noteid, int drugid, sqlite *sql)
               : QDialog(parent, 0, TRUE)
{
    QVBoxLayout *vbox = new QVBoxLayout(this);
    vbox->setMargin(5);
    vbox->setSpacing(3);
    drugLabel = new QLabel(this);
    vbox->addWidget(drugLabel);
    drugLabel->hide();
    QHBoxLayout *hbox = new QHBoxLayout(vbox);
    hbox->addWidget(new QLabel("Title: ", this));
    titleLine = new QLineEdit(this);
    hbox->addWidget(titleLine);
    noteEdit = new QMultiLineEdit(this);
    vbox->addWidget(noteEdit);
    QFrame *sep = new QFrame(this);
    sep->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    vbox->addWidget(sep);
    hbox = new QHBoxLayout(vbox);
    QPushButton *save = new QPushButton("Save", this);
    connect(save, SIGNAL(clicked()), SLOT(slotSaveNote()));
    hbox->addWidget(save);
    deleteNote = new QPushButton(Resource::loadPixmap("trash"),"Delete Note", this);
    connect(deleteNote, SIGNAL(clicked()), SLOT(slotDeleteNote()));
    hbox->addWidget(deleteNote);

    deleteNote->hide();
    sqld=sql;
    drugId=drugid;
    noteId=noteid;
    edit=FALSE;
    
    if( noteId >= 0 || drugId >= 0 ) 
    {
        QString query;
        if(noteId >=0 && drugId < 0) //it's a general note
            query="SELECT title,note FROM notes WHERE num="+QString::number(noteId);
        else if(drugId >=0)  //it's a note about drug
	{
           query="SELECT notes.title,notes.note,drugs.name FROM notes,drugs WHERE notes.drugId="+QString::number(drugId)+"and drugs.id="+QString::number(drugId);
	   setCaption("Create Drug Note");
        }
        char *errmsg;	
        if( sqlite_exec(sql, query, loadDrugNoteCallback,this,&errmsg) != SQLITE_OK )
            QMessageBox::warning(this,"TxDrug","Error reading the database:\n"+QString(errmsg),0);
    }else
        setCaption("Create General Note");

#if defined(Q_WS_QWS) || defined(_WS_QWS_)
    showMaximized();
#endif
     int ret = exec();
     if(ret==QDialog::Accepted)
        slotSaveNote();
}

void NoteDialog::setEdit(bool editing, const char *title, const char *note, const char *drug)
{
    edit=editing;
    if(edit)
      setCaption("Note");
    if(!edit)
        return;
    titleLine->setText(title);
    noteEdit->setText(note);
    if(drug)
    {
        drugLabel->setText("Drug: "+QString(drug));
	drugLabel->show();
    }
    deleteNote->show();
}

void NoteDialog::slotSaveNote()
{
    char *errmsg;
        QString titlestr=titleLine->text();
        if(titlestr.isEmpty())
            titlestr="[Untitled]";
	if(edit)
	{
	    QString field, value;
	    field= noteId >=0 && drugId < 0 ? "num" : "drugId";
            value= noteId >=0 && drugId < 0 ? QString::number(noteId) : QString::number(drugId); 
	    sqlite_exec(sqld,"UPDATE notes SET title='"+titlestr+"' WHERE "+field+" ="+value,
                         0,0,&errmsg);
	    sqlite_exec(sqld,"UPDATE notes SET note='"+noteEdit->text()+"' WHERE "+field+" ="+value,
                         0,0,&errmsg);
	}else
            sqlite_exec(sqld,"INSERT INTO notes (title, note, drugId) values('"+titlestr+"','"+noteEdit->text()+"',"+QString::number(drugId)+")",
                         0,0,&errmsg);
    done(2);
}

void NoteDialog::slotDeleteNote()
{
    bool r = QPEMessageBox::confirmDelete(this,"TxDrug",titleLine->text());
    if(!r)
        return;
    
    QString field, value;
    field= noteId >=0 && drugId < 0 ? "num" : "drugId";
    value= noteId >=0 && drugId < 0 ? QString::number(noteId) : QString::number(drugId); 
    char *errmsg;
    if( sqlite_exec(sqld, "DELETE FROM notes WHERE "+field+" ="+value, 
                         0, 0, &errmsg) != SQLITE_OK)
    {
        QMessageBox::warning(this,"TxDrug","Error deleting the note:\n"+QString(errmsg),0);
    }
    done(2);
}

NotePopup::NotePopup(QWidget *parent,sqlite *sqlc)
	: QPopupMenu(parent)
{
	QAction *a = new QAction(tr( "New Note" ), Resource::loadPixmap("new"),
                              QString::null, 0,0  );
	connect(a, SIGNAL(activated()), parent, SLOT(slotCreateNote()));
	a->addTo(this);
	insertSeparator();
	
	generalNotes = new QPopupMenu(parent);
	insertItem("General Notes",generalNotes);
	drugsNote = new QPopupMenu(this);
	insertItem("Drugs Notes",drugsNote);
	sql=sqlc;
	loadNote();
	changed=false;
}

void NotePopup::loadNote()
{
	uint count=((QMenuData *)this)->count();
	qWarning("count=%d",count);
	drugsNote->clear();
	generalNotes->clear();
	
	char *errmsg;
	sqlite_exec(sql,"SELECT title,drugId,num FROM notes ORDER BY num DESC",
	                   loadNoteCallback,this,&errmsg);
}

void NotePopup::addDrugNote(int id, const char *title)
{
    QString query;
    query.sprintf("SELECT name,id FROM drugs WHERE id=%d", id);
    noteTitle=title;
    char *errmsg;
    if( sqlite_exec(sql, query, drugNameCallback, this, &errmsg) != SQLITE_OK )
        QMessageBox::warning(this,"TxDrug","Error loading notes:\n"+QString(errmsg),0); 
    delete errmsg;
}

void NotePopup::slotDeleteNote()
{
    if ( !QPEMessageBox::confirmDelete( 0,tr( "Note" ), titleline->text()  ) )
        return;

    char *errmsg;
    sqlite_exec(sql,"DELETE FROM notes WHERE title='"+titleline->text()+"'",0,0,&errmsg);
    delete d;
    loadNote();
}

void NotePopup::slotNote(int noteId)
{
    qWarning("in slotNote noteid=%d", noteId);
    (void)new NoteDialog(parentWidget(), noteId, -1, sql);
    loadNote();
}

void NotePopup::slotDrugNote(int drugId)
{
    (void)new NoteDialog(parentWidget(), -1, drugId, sql);
    loadNote();
}


